# This is the main shinyTex function.  It takes the name of a shinyTex .tex file and
# creates ui.R, server.R, and various #-#.html files.  Together these are a Shiny app.
#
shinyTex = function(fname=NULL, cleanFirst=TRUE) {
  #if (!require("shiny")) stop("please install R package 'shiny'")
  #if (!require("stringr")) stop("please install R package 'stringr'")
  
  # preliminary steps
  if (is.null(fname)) fname = guessFilename()
  fname = checkFile(fname, ending=".tex")
  if (cleanFirst) cleanFiles()

  # read .tex file, make validity checks, and identify structure
  tex = readLines(fname)
  texInfo = parseTex(tex, fname)
  tex = texInfo$cleanTex
  
  # Start list of what needs to be done
  doList = startDoList(tex, texInfo$defs, texInfo$tabTitles, texInfo$mainStartEnd, 
                       texInfo$tabbing)

  # Loop to required depth (storing global information for error messaging)
  globalInfo = new.env()
  assign("lineNumOffset", texInfo$lineNumOffset, envir=globalInfo)
  assign("tabTitles", texInfo$tabTitles, envir=globalInfo)
  toDo = sapply(doList, function(x) is.numeric(x$uiCode))
  while (any(toDo)) {
    doList = downOne(doList, toDo, tex, globalInfo)
    toDo = sapply(doList, function(x) is.numeric(x$uiCode))
  }

  # Look for "large scale" errors
  finalChecks(globalInfo)

  # Final fixups
  if (texInfo$tabbing) {
    doList[[1]]$serverCode = c(doList[[1]]$serverCode, addTabObservers(globalInfo))
  }
  doList = dropExtraCommas(doList)
  
  # Write UI and server code
  index = 1
  while (TRUE) {
    this = doList[[index]]
    writeUi(this$uiCode, indent=this$indent, new=(index==1))
    if (!is.na(this$serverCode[1])) {
      writeServer(this$serverCode, indent=as.numeric(index>1 && !is.na(this$nextElement)), new=(index==1))
    }
    index = this$nextElement
    if (is.na(index)) break
  } 

  cat("Successfully created Shiny files for your '", texInfo$defs$title, "' app.", sep="")
  return(invisible(NULL))
}
